/*
 * linux/drivers/char/aj_led.c
 *
 * Copyright (C) 2004 Atmark Techno, Inc.
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <asm/uaccess.h>
#include <asm/io.h>

#define AJ_LED_MAJOR	240
#define CHRDEV			"aj_led"
#define VERSION			CHRDEV ": armadillo-j led control driver, (C) 2004 Atmark Techno, Inc."

#define AJ_LED_CTRL_REG	(0xFFB00028) 
#define AJ_LED_DEV_CHR	"ajled"

#define		LED_BASE	(0xFFB00028) 
#define		LED_STS		((0x20))

#define		LED_MODE( ) ( ~((LED_STS)<<24) ) 
#define		LED_DIR( )	(  ((LED_STS)<<16) ) 
#define		LED_SF( )	( ~((LED_STS)<<8) ) 
#define		LED_ON( )	( ~((LED_STS)) ) 
#define		LED_OFF( )	( ((LED_STS)) ) 
#define		LED_DATA( val ) ( ((LED_STS)&(val))>>5 ) 

#define		PUTOFF_LED (1)
#define		PUTON_LED (0)

#define DBGMODE 1
#undef DBGMODE 

#ifdef DBGMODE 
#define PRT( args... ) printk( KERN_CRIT args ) 
#define LINE( ) PRT( "line:%d\n", __LINE__ ) 
#else
#define PRT( args... )
#define LINE( args... )
#endif /* DBGMODE */ 

inline static unsigned long get_register(void)
{
    return *(volatile unsigned long*)AJ_LED_CTRL_REG;
}

inline static void set_register(unsigned long val)
{
    *(volatile unsigned long*)AJ_LED_CTRL_REG = val;
}

/****************************************************************

****************************************************************/
static ssize_t aj_led_read (
  struct file *file,
  char *buf,
  size_t count,
  loff_t *ppos
)
{
  char tmp;
  int val ; 
  tmp = LED_DATA( get_register( ) ) ; 
  PRT( "data:0x%08X\n", tmp ) ; 

  if( tmp == PUTOFF_LED ) { 
    val = 0 ; 
  } else if ( tmp == PUTON_LED ) { 
    val = 1 ; 
  } else { 
    LINE( ) ; 
    return -1 ; 
  } 

  if (put_user (val, buf)) { 
    LINE( ) ; 
    return -EFAULT;
  } 
  return 1;
}

/****************************************************************

****************************************************************/
static ssize_t aj_led_write (
  struct file *file,
  const char *buf,
  size_t count,
  loff_t *ppos
)
{
  unsigned long val = 0 ;
  if( *buf == 0 ) { /* put off led */ 
    val &= ((LED_MODE())&(LED_SF())) ; 
    val |= (LED_DIR()|LED_OFF()) ; 

  } else { /* put on led */ 
    LINE( ) ; 
    PRT( "data:0x%02X\n", *buf ) ; 
    val &= ((LED_MODE())&(LED_SF())&(LED_ON())) ; 
    val |= LED_DIR() ; 
  } 
  set_register( val ) ; 

  return 1;
}

/****************************************************************

****************************************************************/
static struct file_operations aj_led_fops = {
  owner: THIS_MODULE,
  read:  aj_led_read,
  write: aj_led_write,
};

/****************************************************************

****************************************************************/
static int __init aj_led_init (void)
{
  if (!request_region (AJ_LED_CTRL_REG, 1, AJ_LED_DEV_CHR)) {
    printk (KERN_WARNING CHRDEV ": I/O port 0x%x is not free.\n",
     AJ_LED_CTRL_REG);
    return -EIO;
  }
  
  if (register_chrdev (AJ_LED_MAJOR, CHRDEV, &aj_led_fops)) {
    printk (KERN_WARNING CHRDEV ": Unable to get major %d.\n", AJ_LED_MAJOR);
    return -EIO;
  }
  
  printk (KERN_INFO VERSION "\n");
  return 0;
}

/****************************************************************

****************************************************************/
static void __exit aj_led_cleanup (void)
{
  unregister_chrdev (AJ_LED_MAJOR, CHRDEV);
  
  release_region (AJ_LED_CTRL_REG, 1);
}

module_init (aj_led_init);
module_exit (aj_led_cleanup);
